@@ -2,7 +2,7 @@
 
 /**
   * SquirrelMail Login Manager Plugin
-  * Copyright (c) 2002-2008 Paul Lesniewski <paul@squirrelmail.org>
+  * Copyright (c) 2002-2009 Paul Lesniewski <paul@squirrelmail.org>
   * Copyright (c) 2001 Ryan Orth <snubber-sm@seahat.com>
   * Licensed under the GNU GPL. For full terms see the file COPYING.
   *
@@ -24,7 +24,7 @@ function vlogin_check_configuration_do()
 {
 
    global $useSessionBased, $allVirtualDomainsAreUnderOneHost, 
-          $virtualUserTableDBFlavor;
+          $virtualUserTableDBFlavor, $sq_ignore_http_x_forwarded_headers;
 
 
    // make sure there is some config out there
@@ -109,7 +109,9 @@ function vlogin_check_configuration_do()
          $_SERVER = $HTTP_SERVER_VARS;
       }
 
-      if (isset($_SERVER['HTTP_X_FORWARDED_HOST']) && !empty($_SERVER['HTTP_X_FORWARDED_HOST']))
+      if (!$sq_ignore_http_x_forwarded_headers
+       && isset($_SERVER['HTTP_X_FORWARDED_HOST'])
+       && !empty($_SERVER['HTTP_X_FORWARDED_HOST']))
          $hostname = $_SERVER['HTTP_X_FORWARDED_HOST'];
       else
          $hostname = $_SERVER['HTTP_HOST'];
@@ -206,8 +208,9 @@ function load_vlogin_config()
    }
 
 
-   // load custom config if present, or load typical settings 
-   // if none found at all
+   // load custom config if present, or load typical (config_default
+   // is currently a copy of config_example_typical) settings if
+   // none found at all
    //
    // Note: could code all these if's into one single test with all &&'s
    //       but it is kept this way for clarity; won't change unless
@@ -216,7 +219,7 @@ function load_vlogin_config()
    if (!@include_once(SM_PATH . 'config/config_vlogin/config.php'))
       if (!@include_once(SM_PATH . 'plugins/vlogin/data/config.php'))
          if (!$config_settings_loaded)
-            if (!@include_once(SM_PATH . 'plugins/vlogin/data/config.sample.typical.php'))
+            if (!@include_once(SM_PATH . 'plugins/vlogin/data/config_default.php'))
                return FALSE;
 
    return TRUE;
@@ -264,7 +267,7 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
           $replacements, $removeFromFront, $translateHostnameTable, 
           $pathToQmail, $reverseDotSectionOrder,
           $always_prepend, $always_append, $IMAPServerRules, 
-          $SMTPServerRules;
+          $SMTPServerRules, $sq_ignore_http_x_forwarded_headers;
       
   
   // debugging/performance tuning
@@ -413,7 +416,9 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
 
      // grab hostname into local var
      //
-     if (isset($_SERVER['HTTP_X_FORWARDED_HOST']) && !empty($_SERVER['HTTP_X_FORWARDED_HOST']))
+     if (!$sq_ignore_http_x_forwarded_headers
+      && isset($_SERVER['HTTP_X_FORWARDED_HOST'])
+      && !empty($_SERVER['HTTP_X_FORWARDED_HOST']))
         $hostname = $_SERVER['HTTP_X_FORWARDED_HOST'];
      else
         $hostname = $_SERVER['HTTP_HOST'];
@@ -453,18 +458,6 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
      $orig_hostname_stripped = $hostname_stripped;
    
    
-     // set domain if the $smHostIsDomainThatUserLoggedInWith
-     // flag is on...
-     //
-     if ($smHostIsDomainThatUserLoggedInWith)
-     {
-        if ($useSessionBased)
-           $config_override['domain'] = $hostname_stripped;
-        else
-           $domain = $hostname_stripped; 
-     }
-      
-
      // allow domain given in username to override host
      // if needed - yikes
      //
@@ -486,6 +479,18 @@ function overrideSmConfig($args, $user='', $hostname='', $hostname_stripped='',
      }
 
 
+     // set domain if the $smHostIsDomainThatUserLoggedInWith
+     // flag is on...
+     //
+     if ($smHostIsDomainThatUserLoggedInWith)
+     {
+        if ($useSessionBased)
+           $config_override['domain'] = $hostname_stripped;
+        else
+           $domain = $hostname_stripped; 
+     }
+      
+
      // determine IMAP server address if it should be based 
      // on the user's domain
      //
@@ -951,7 +956,8 @@ function determine_user_hostname()
           $chopOffDotSectionsFromLeft, $numberOfDotSections, 
           $checkByExcludeList, $at, $dot, $replacements,
           $removeFromFront, $translateHostnameTable, $pathToQmail, 
-          $reverseDotSectionOrder, $always_prepend, $always_append;
+          $reverseDotSectionOrder, $always_prepend, $always_append,
+          $sq_ignore_http_x_forwarded_headers;
 
 
    // get vlogin config settings
@@ -973,7 +979,9 @@ function determine_user_hostname()
 
    // grab hostname into local var
    //
-   if (isset($_SERVER['HTTP_X_FORWARDED_HOST']) && !empty($_SERVER['HTTP_X_FORWARDED_HOST']))
+   if (!$sq_ignore_http_x_forwarded_headers
+    && isset($_SERVER['HTTP_X_FORWARDED_HOST'])
+    && !empty($_SERVER['HTTP_X_FORWARDED_HOST']))
       $hostname = $_SERVER['HTTP_X_FORWARDED_HOST'];
    else
       $hostname = $_SERVER['HTTP_HOST'];
@@ -1121,7 +1129,7 @@ function vlogin_domain_do($args)
          $prefs_are_cached, $prefs_cache, $vlogin_remap_username_completed,
          $dontUseHostNameUserList, $postProcessingReplacement, 
          $postProcessingPattern, $virtualUserTableDBFlavor,
-         $override_config_during_login;
+         $override_config_during_login, $serviceLevel;
 
 
   // debugging/performance tuning
@@ -1349,6 +1357,18 @@ function vlogin_domain_do($args)
   }
 
 
+  // override settings for different service levels
+  //
+  if (serviceLevelOverride($realname) || !empty($serviceLevel))
+  {
+     // if we don't give other args to this, it will take
+     // take care of finding any service level overrides
+     // in SQL database if needed
+     //
+     overrideSmConfig($args, $realname);
+  }
+
+
   // override settings on a per-user basis
   //
   perUserOverride($realname, $args);
@@ -2073,101 +2093,107 @@ function getVirtualUserFromDB($user, $virtualUserDBTable, $virtualUserTableDBFla
 function vlogin_display_options_do()
 {
 
-   global $config_override, $optpage_data, $notPartOfDomainName, 
-          $chopOffDotSectionsFromRight, $chopOffDotSectionsFromLeft, 
-          $numberOfDotSections, $checkByExcludeList, $at, $dot, 
-          $replacements, $removeFromFront, $translateHostnameTable, 
-          $pathToQmail, $reverseDotSectionOrder, $always_prepend, 
-          $always_append;
+   global $disable_option_pages, $optpage;
 
-   $overrides = array();
-
-
-   // try to get configuration settings from session, if they're there
+   // make sure user isn't trying to access/submit
+   // a disallowed option page
    //
-   if (!isset($config_override))
+   // 0 = Personal Information = personal
+   // 1 = Display Preferences  = display
+   // 2 = Message Highlighting = highlight
+   // 3 = Folder Preferences   = folder
+   // 4 = Index Order          = order
+   // 5 = Compose Preferences  = compose
+   //
+   if (isset($disable_option_pages) && is_array($disable_option_pages))
    {
 
-      global $virtualDomains, $useSessionBased;
-      load_vlogin_config();
-
-      if (!$useSessionBased)
+      switch ($optpage)
       {
-
-         // get global variable for versions of PHP < 4.1 
-         // doing this instead of sqGetGlobalVar() because
-         // this code might be called before that function
-         // is defined (TODO: as of 1.5.2, this may no longer
-         // be true)
-         //
-         if (!check_php_version(4,1)) 
-         {
-            global $HTTP_SERVER_VARS;
-            $_SERVER = $HTTP_SERVER_VARS;
-         }
-
-
-         // grab hostname into local var
-         //
-         if (isset($_SERVER['HTTP_X_FORWARDED_HOST']) && !empty($_SERVER['HTTP_X_FORWARDED_HOST']))
-            $hostname = $_SERVER['HTTP_X_FORWARDED_HOST'];
-         else
-            $hostname = $_SERVER['HTTP_HOST'];
-         $hostname_stripped = deconstructDomainName($hostname,
-                                                    $notPartOfDomainName, 
-                                                    $chopOffDotSectionsFromRight, 
-                                                    $chopOffDotSectionsFromLeft, 
-                                                    $numberOfDotSections, 
-                                                    $checkByExcludeList, 
-                                                    $at, $dot, $replacements,
-                                                    $removeFromFront, 
-                                                    $translateHostnameTable, 
-                                                    $pathToQmail,
-                                                    $reverseDotSectionOrder,
-                                                    $always_prepend, $always_append);
-
-
-         // allow inclusion of external configuration files for each domain
-         //
-         if (file_exists(SM_PATH . 'config/config_vlogin/domains/' . $hostname_stripped . '.vlogin.config.php'))
-            include_once(SM_PATH . 'config/config_vlogin/domains/' . $hostname_stripped . '.vlogin.config.php');
-         else if (file_exists(SM_PATH . 'plugins/vlogin/data/domains/' . $hostname_stripped . '.vlogin.config.php'))
-            include_once(SM_PATH . 'plugins/vlogin/data/domains/' . $hostname_stripped . '.vlogin.config.php');
-
-
-         // get virtual domain config items
-         //
-         $overrides = array();
-         if (is_array($virtualDomains))
-         foreach (array_keys($virtualDomains) as $virtualDomain) 
-         {
-
-            if (stristr($hostname, (string)($virtualDomain)) || $virtualDomain == '*') 
-            {
-
-               $overrides = $virtualDomains[$virtualDomain];
-
-            }
-
-         }
-
+         case 'personal':
+            if (in_array(0, $disable_option_pages))
+               exit;  // don't print anything out to help attackers
+            break;
+         case 'display':
+            if (in_array(1, $disable_option_pages))
+               exit;  // don't print anything out to help attackers
+            break;
+         case 'highlight':
+            if (in_array(2, $disable_option_pages))
+               exit;  // don't print anything out to help attackers
+            break;
+         case 'folder':
+            if (in_array(3, $disable_option_pages))
+               exit;  // don't print anything out to help attackers
+            break;
+         case 'order':
+            if (in_array(4, $disable_option_pages))
+               exit;  // don't print anything out to help attackers
+            break;
+         case 'compose':
+            if (in_array(5, $disable_option_pages))
+               exit;  // don't print anything out to help attackers
+            break;
       }
 
    }
-   else
-      $overrides = $config_override;
-
 
+   global $disable_options, $optpage_data;
 
    // remove any options from list of options as desired
    //
-   if (isset($overrides['disable_options']) && is_array($overrides['disable_options']))
-      foreach ($overrides['disable_options'] as $removeOptionName)
+   if (isset($disable_options) && is_array($disable_options))
+   {
+      foreach ($disable_options as $removeOptionName)
          foreach ($optpage_data['vals'] as $groupNumber => $optionGroup)
             foreach ($optionGroup as $optionNumber => $optionItem)
                if (isset($optionItem['name']) && $optionItem['name'] == $removeOptionName)
                   unset($optpage_data['vals'][$groupNumber][$optionNumber]);
 
+
+      // check if no other elements are left in the option group
+      // (except hidden elements)...  remove the whole group if
+      // so (are there any hidden elements that we WANT to retain??)
+      //
+      foreach ($optpage_data['vals'] as $groupNumber => $optionGroup)
+      {
+         $group_is_empty = TRUE;
+         foreach ($optionGroup as $optionNumber => $optionItem)
+            if (isset($optionItem['type']) && $optionItem['type'] != SMOPT_TYPE_HIDDEN)
+            {
+               $group_is_empty = FALSE;
+               break;
+            }
+         if ($group_is_empty)
+         {
+            unset($optpage_data['vals'][$groupNumber]);
+            unset($optpage_data['grps'][$groupNumber]);
+         }
+      }
+   }
+
+}
+
+
+
+/**
+  * Allows control over what option blocks (pages)
+  * are available on main options page
+  *
+  */
+function vlogin_option_block_do()
+{
+
+   global $disable_option_pages, $optpage_blocks;
+
+   // remove any option blocks as needed
+   //
+   if (isset($disable_option_pages) && is_array($disable_option_pages))
+   {
+      foreach ($disable_option_pages as $page)
+         unset($optpage_blocks[$page]);
+   }
+
 }
 
 
